package org.expeditee.gui;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;

import org.expeditee.items.Item;

//Manages slide animations between frames
//Reflection is used to call these methods, therefore any additions should only need to be added in this class.
//No further changes to other code should be required.
public class FrameTransitions {	
	
	//Stores the frame before the new frame being transitioned to
	private static Image _slideModeBaseImage = null;
	private static Image _slideModeScratchImage = null;
	
	//Tells graphics which method of transition is going to be used
	private static String _slideModeMethod;
	
	//Used for power point slide effects
	private static boolean slide = false;
		
	public static Boolean getSlide(){
		
		return slide;
	}
	public static void setSlideFalse(){
		
		slide = false;
	}
	public static void setSlideTrue(){
		
		slide = true;
	}
	public static void setSlideModeBaseImage(Image i){
		
		_slideModeBaseImage = i;
		_slideModeScratchImage = new BufferedImage(i.getWidth(null), i.getHeight(null), BufferedImage.TYPE_INT_ARGB);
	}
	public static Image getSlideModeBaseImage(){
		
		return _slideModeBaseImage;		
	}
	public static void setSlideModeMethod(String s){
		
		_slideModeMethod = s;
	}
	public static String getslideModeMethod(){
		
		return _slideModeMethod;
	}
	
	//Slides new frame over current from from Right to Left
	public static void Swipe(Graphics g, Image i, Dimension d){
		
		Image image = i;
		
		int frameX = d.width;
		
		while(frameX != 0){
			
			g.drawImage(image, frameX, 0, Item.DEFAULT_BACKGROUND, null);
								
			frameX--;								
		}	
	}
	//Drops new frame from top of display
	//Draws rectangle with gradient effect to create illusion of drop shadow	
	public static void FallDown(Graphics g, Image i, Dimension d){
		
		Graphics2D g2 = (Graphics2D) g;
	    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
	    RenderingHints.VALUE_ANTIALIAS_ON);

	    Image image = i;		
		int frameY = 0 - d.height;
		int vertCount = 0;
		
		while(frameY != 0){
			
			GradientPaint shadow = new GradientPaint(0, vertCount + 10, Color.WHITE, 0, vertCount, Color.gray);
		    g2.setPaint(shadow);
			g.drawImage(image,  0,  frameY,  Item.DEFAULT_BACKGROUND,  null);	
			g2.fillRect(0, vertCount, d.width, 10);		
			
			frameY++;
			vertCount++;
			try {
				Thread.sleep(3);
			} catch (InterruptedException e) {
			
				e.printStackTrace();
			}			
		}        
	}		

	//Use synchronized to ensure no issues with concurrent update issues	
	//Fades out current frame to black, fades new slide in from black
	public static void FadeInBlack(Graphics g, Image i, Dimension d){		
	
		String fadeType = "black";
		float alpha = 1.0f;		
		Graphics2D g2d = (Graphics2D) g;		
		
		while(alpha >= 0.3f){
			
			g.drawImage(FrameTransitions.getSlideModeBaseImage(),  0,  0,  Item.DEFAULT_BACKGROUND,  null);			
			g2d.setComposite(alphaCalcUp(alpha, fadeType));				
			g2d.fillRect(0, 0, d.width, d.height);
			alpha = alphaCalcUp(alpha, fadeType).getAlpha();
			
			try {
				Thread.sleep(30);
			} catch (InterruptedException e) {
			
				e.printStackTrace();
			}			
		}		
		while(alpha <= 0.8f){			
		
			g.drawImage(i,  0,  0,  Item.DEFAULT_BACKGROUND,  null);
			g2d.setComposite(alphaCalcDown(alpha, fadeType));			
			g2d.fillRect(0, 0, d.width, d.height);
			alpha = alphaCalcDown(alpha, fadeType).getAlpha();
			
			try {
				Thread.sleep(30);
			} catch (InterruptedException e) {

				e.printStackTrace();
			}	
		}       
	}
	//Fades current frame out, and new frame in
	//Calls AlphaCalcUp and AlphaCalcDown to calculate Alpha values of these transitions
	//AlphaComposite is passed in as a parameter, and it holds the alpha value
	public static void FadeInWhite(Graphics g, Image i, Dimension d){
		
		String fadeType = "white";
		float alpha = 1.0f;
		
		Graphics2D g2d = (Graphics2D) g;		
		
		Graphics2D scratch_g2d = (Graphics2D)_slideModeScratchImage.getGraphics();
		
		while(alpha >= 0.1f){			
			
			
			scratch_g2d.drawImage(getSlideModeBaseImage(),  0,  0,  Item.DEFAULT_BACKGROUND,  null);			
			scratch_g2d.setComposite(alphaCalcUp(alpha, fadeType));				
			scratch_g2d.setColor(Color.WHITE);
			scratch_g2d.fillRect(0, 0, d.width, d.height);
			
			g2d.drawImage(_slideModeScratchImage, 0, 0, null);
			
			alpha = alphaCalcUp(alpha, fadeType).getAlpha();
			
			try {
				Thread.sleep(25);
			} catch (InterruptedException e) {
			
				e.printStackTrace();
			}			
			
			System.out.println(alpha);
		}		
		while(alpha <= 0.75f){			
		
			scratch_g2d.drawImage(i,  0,  0,  Item.DEFAULT_BACKGROUND,  null);
			scratch_g2d.setComposite(alphaCalcDown(alpha, fadeType));
			scratch_g2d.setColor(Color.WHITE);
			scratch_g2d.fillRect(0, 0, d.width, d.height);
		
			g2d.drawImage(_slideModeScratchImage, 0, 0, null);
			
			alpha = alphaCalcDown(alpha, fadeType).getAlpha();
			
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {

				e.printStackTrace();
			}	
			
			System.out.println(alpha);
		} 
	}

	//Calculates alpha values for transitions from transparent to opaque
	//AlphaComposite is returned
	private static AlphaComposite alphaCalcUp(float i, String fadeType){
		
		float alpha = i;
		float increment = -0.05f;
		alpha += increment;		

		if(fadeType == "black"){
		
			AlphaComposite alcom = AlphaComposite.getInstance(AlphaComposite.DST_IN, alpha);
			return alcom;		
		}
		else{
			//fadeType White
			AlphaComposite alcom = AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, alpha);
			return alcom;
		}
	}
	//Calculates alpha values for transitions from opaque to transparent	
	//AlphaComposite is returned
	private static AlphaComposite alphaCalcDown(float i, String fadeType){

		float alpha = i;
		float increment = -0.05f;
		alpha -= increment;		

		if(fadeType == "black"){
		
			AlphaComposite alcom = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha);
			return alcom;		
		}
		else{
			//fadeType White
			//AlphaComposite alcom = AlphaComposite.SrcOver.derive(alpha);
			AlphaComposite alcom = AlphaComposite.getInstance(AlphaComposite.SRC_ATOP, alpha);
			return alcom;
		}
	}
	
	public static void CrossFade(Graphics g, Image i, Dimension d)
	{ 
		// Based on: http://stackoverflow.com/questions/20346661/java-fade-in-and-out-of-images
		// TODO: make use of timer, as this example does, rather than fixed delay
		
		Image inImage = i;
		Image outImage = _slideModeBaseImage;
		int scratch_x_dim = _slideModeScratchImage.getWidth(null);
		int scratch_y_dim = _slideModeScratchImage.getHeight(null);

		int anim_timestep_msec = 10;
		float alpha = 0.0f;
		float alpha_inc = 0.04f;
		
		Graphics2D g2d = (Graphics2D) g;		

		Graphics scratch_g = _slideModeScratchImage.getGraphics();

		while(alpha <= 1.0f){			

			//scratch_g.clearRect(0, 0, scratch_x_dim, scratch_y_dim);
			scratch_g.setColor(Color.WHITE);
			scratch_g.fillRect(0, 0, scratch_x_dim, scratch_y_dim);
			
			Graphics2D scratch_g2d = (Graphics2D) scratch_g.create();
			scratch_g2d.setComposite(AlphaComposite.SrcOver.derive(alpha));
			scratch_g2d.drawImage(inImage, 0, 0, null);

			scratch_g2d.setComposite(AlphaComposite.SrcOver.derive(1f - alpha));
			scratch_g2d.drawImage(outImage, 0, 0, null);
			scratch_g2d.dispose();

			g2d.drawImage(_slideModeScratchImage, 0, 0, null);

			try {
				Thread.sleep(anim_timestep_msec);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}			

			alpha += alpha_inc;	
		}		
	}	

	
}
